=== modified file 'NEWS'
--- NEWS	2009-03-04 18:11:47 +0000
+++ NEWS	2009-07-28 04:56:32 +0000
@@ -3,6 +3,8 @@
 
 Improvements
 ------------
+ - Add support for latest version on Django by not checking arguments
+   passed to _cursor.
  - Added support for using Expressions with ResultSet.set().
  - The default cache size was changed from 100 to 1000 objects.
  - Added the new GenerationalCache with a faster implementation of
@@ -14,6 +16,21 @@
  - The StoreDataManager now gets passed a transaction manager from the
    ZStorm utility.  This will make it easier to support non-default
    transaction managers in the future.
+ - An adapter is now available for converting ISQLObjectResultSet
+   objects to IResultSet.  This is intended to help in gradual porting
+   SQLObject applications to Storm's native API (bug #338184, by Gavin
+   Panella).
+ - If a disconnection occurs outside of Storm's control, a
+   DisconnectionError will still be raised next time the connection is
+   used through Storm.  This is useful if the connection is being
+   shared with another framework like Django (bug #374909).
+ - The PostgreSQL backend now requires psycopg2 >= 2.0.7.  The work
+   around for broken quoting behaviour in older psycopg2 versions has
+   been removed. (bug #322206).
+ - A new Neg expression is available.  It provides unary minus by
+   prepending a minus sign to whatever expression is passed to it
+   (bug #397654 by Michael Hudson).
+ - A new Coalese expressions is available.
 
 Bug fixes
 ---------
@@ -22,6 +39,8 @@
  - The test suite now passes when run with Python 2.6.
  - ListVariable now converts its elements to database representation
    correctly (bug #136806 reported by Marc Tardif).
+ - compile_python now works for values that don't produce valid Python
+   expressions with repr().
 
 
 0.14 (2009-01-09)

=== modified file 'storm/database.py'
--- storm/database.py	2008-09-30 11:45:14 +0000
+++ storm/database.py	2009-05-15 08:43:21 +0000
@@ -28,7 +28,8 @@
 from storm.expr import Expr, State, compile
 from storm.tracer import trace
 from storm.variables import Variable
-from storm.exceptions import ClosedError, DatabaseError, DisconnectionError
+from storm.exceptions import (
+    ClosedError, DatabaseError, DisconnectionError, Error)
 from storm.uri import URI
 import storm
 
@@ -273,9 +274,10 @@
 
         @return: The dbapi cursor object, as fetched from L{build_raw_cursor}.
         """
-        raw_cursor = self.build_raw_cursor()
-        trace("connection_raw_execute", self, raw_cursor,
-              statement, params or ())
+        raw_cursor = self._check_disconnect(self.build_raw_cursor)
+        self._check_disconnect(
+            trace, "connection_raw_execute", self, raw_cursor,
+            statement, params or ())
         if params:
             args = (statement, tuple(self.to_database(params)))
         else:
@@ -283,12 +285,14 @@
         try:
             self._check_disconnect(raw_cursor.execute, *args)
         except Exception, error:
-            trace("connection_raw_execute_error", self, raw_cursor,
-                  statement, params or (), error)
+            self._check_disconnect(
+                trace, "connection_raw_execute_error", self, raw_cursor,
+                statement, params or (), error)
             raise
         else:
-            trace("connection_raw_execute_success", self, raw_cursor,
-                  statement, params or ())
+            self._check_disconnect(
+                trace, "connection_raw_execute_success", self, raw_cursor,
+                statement, params or ())
         return raw_cursor
 
     def _ensure_connected(self):
@@ -323,7 +327,7 @@
         """Run the given function, checking for database disconnections."""
         try:
             return function(*args, **kwargs)
-        except DatabaseError, exc:
+        except Error, exc:
             if self.is_disconnection_error(exc):
                 self._state = STATE_DISCONNECTED
                 self._raw_connection = None

=== modified file 'storm/databases/postgres.py'
--- storm/databases/postgres.py	2008-08-18 19:44:51 +0000
+++ storm/databases/postgres.py	2009-07-07 02:06:59 +0000
@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2006, 2007 Canonical
+# Copyright (c) 2006-2009 Canonical
 #
 # Written by Gustavo Niemeyer <gustavo@niemeyer.net>
 #
@@ -19,13 +19,23 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 from datetime import datetime, date, time, timedelta
+from distutils.version import LooseVersion
 
 from storm.databases import dummy
 
+# PostgreSQL support in Storm requires psycopg2 2.0.7 or greater.
+# Earlier versions of psyocpg2 contain data loss bugs.
+# See https://bugs.launchpad.net/storm/+bug/322206 for more details.
+REQUIRED_PSYCOPG2_VERSION = LooseVersion('2.0.7')
+PSYCOPG2_VERSION = None
 try:
     import psycopg2
-    import psycopg2.extensions
-except:
+    PSYCOPG2_VERSION = LooseVersion(psycopg2.__version__.split(' ')[0])
+    if PSYCOPG2_VERSION < REQUIRED_PSYCOPG2_VERSION:
+        psycopg2 = dummy
+    else:
+        import psycopg2.extensions
+except ImportError:
     psycopg2 = dummy
 
 from storm.expr import (
@@ -33,10 +43,10 @@
     Sequence, Like, SQLToken, COLUMN, COLUMN_NAME, COLUMN_PREFIX, TABLE,
     compile, compile_select, compile_insert, compile_set_expr, compile_like,
     compile_sql_token)
-from storm.variables import Variable, ListVariable, RawStrVariable
+from storm.variables import Variable, ListVariable
 from storm.database import Database, Connection, Result
 from storm.exceptions import (
-    install_exceptions, DatabaseError, DatabaseModuleError,
+    install_exceptions, DatabaseError, DatabaseModuleError, InterfaceError,
     OperationalError, ProgrammingError, TimeoutError)
 from storm.tracer import TimeoutTracer
 
@@ -193,24 +203,6 @@
     return compile_sql_token(compile, expr, state)
 
 
-def compile_str_variable_with_E(compile, variable, state):
-    """Include an E just before the placeholder of string variables.
-
-    PostgreSQL 8.2 will issue a warning without it, and psycopg
-    will use the plain '' rather than E''.  The problem is being
-    tracked at the following URL::
-
-        http://www.initd.org/tracker/psycopg/ticket/202
-
-    """
-    state.parameters.append(variable)
-    if type(variable.get(to_db=True)) is str:
-        return "E?"
-    return "?"
-
-psycopg_needs_E = None
-
-
 class PostgresResult(Result):
 
     def get_insert_identity(self, primary_key, primary_variables):
@@ -285,17 +277,20 @@
                 yield param
 
     def is_disconnection_error(self, exc):
-        if not isinstance(exc, (OperationalError, ProgrammingError)):
+        if not isinstance(exc, (InterfaceError, OperationalError,
+                                ProgrammingError)):
             return False
 
         # XXX: 2007-09-17 jamesh
-        # I have no idea why I am seeing the last exception message
-        # after upgrading to Gutsy.
+        # The last message is for the benefit of old versions of
+        # psycopg2 (prior to 2.0.7) which have a bug related to
+        # stripping the error severity from the message.
         msg = str(exc)
         return ("server closed the connection unexpectedly" in msg or
                 "could not connect to server" in msg or
                 "no connection to the server" in msg or
                 "connection not open" in msg or
+                "connection already closed" in msg or
                 "losed the connection unexpectedly" in msg)
 
 
@@ -311,7 +306,9 @@
 
     def __init__(self, uri):
         if psycopg2 is dummy:
-            raise DatabaseModuleError("'psycopg2' module not found")
+            raise DatabaseModuleError(
+                "'psycopg2' >= %s not found. Found %s."
+                % (REQUIRED_PSYCOPG2_VERSION, PSYCOPG2_VERSION))
         self._dsn = make_dsn(uri)
         isolation = uri.options.get("isolation", "serializable")
         isolation_mapping = {
@@ -337,29 +334,8 @@
                 cursor.execute("SHOW server_version_num")
             except psycopg2.ProgrammingError:
                 self._version = 0
-                raw_connection.rollback()
             else:
                 self._version = int(cursor.fetchone()[0])
-
-            # This will conditionally change the compilation of binary
-            # variables (RawStrVariable) to preceed the placeholder with an
-            # 'E', if psycopg isn't doing it by itself.
-            #
-            # The "failing" code path isn't unittested because that
-            # would depend on a working psycopg version, or in an older
-            # postgresql version.  Both branches were manually tested
-            # for correctness at some point.
-            global psycopg_needs_E
-            try:
-                # If psycopg behaves correctly, this will break trying
-                # to run a query with EE''.
-                cursor.execute("SELECT E%s", (psycopg2.Binary(""),))
-            except psycopg2.ProgrammingError:
-                psycopg_needs_E = False
-            else:
-                psycopg_needs_E = True
-                compile.when(RawStrVariable)(compile_str_variable_with_E)
-
             raw_connection.rollback()
 
         raw_connection.set_client_encoding("UTF8")

=== modified file 'storm/django/backend/base.py'
--- storm/django/backend/base.py	2008-10-21 09:19:47 +0000
+++ storm/django/backend/base.py	2009-04-17 13:20:26 +0000
@@ -26,8 +26,8 @@
 
     connection = property(_get_connection, _set_connection)
 
-    def _cursor(self, settings):
-        cursor = super(StormDatabaseWrapperMixin, self)._cursor(settings)
+    def _cursor(self, *args):
+        cursor = super(StormDatabaseWrapperMixin, self)._cursor(*args)
         self._store._event.emit("register-transaction")
         return cursor
 

=== modified file 'storm/exceptions.py'
--- storm/exceptions.py	2008-01-31 21:29:55 +0000
+++ storm/exceptions.py	2009-05-15 08:43:21 +0000
@@ -130,7 +130,7 @@
 def install_exceptions(module):
     for exception in (Error, Warning, DatabaseError, InternalError,
                       OperationalError, ProgrammingError, IntegrityError,
-                      DataError, NotSupportedError):
+                      DataError, NotSupportedError, InterfaceError):
         module_exception = getattr(module, exception.__name__, None)
         if module_exception is not None:
             module_exception.__bases__ += (exception,)

=== modified file 'storm/expr.py'
--- storm/expr.py	2008-06-27 16:44:28 +0000
+++ storm/expr.py	2009-07-23 22:47:10 +0000
@@ -206,8 +206,18 @@
 class CompilePython(Compile):
 
     def get_matcher(self, expr):
-        exec "def match(get_column): return bool(%s)" % self(expr)
-        return match
+        state = State()
+        source = self(expr, state)
+        namespace = {}
+        code = ("def closure(parameters, bool):\n"
+                "    [%s] = parameters\n"
+                "    def match(get_column):\n"
+                "        return bool(%s)\n"
+                "    return match" %
+                (",".join("_%d" % i for i in range(len(state.parameters))),
+                 source))
+        exec code in namespace
+        return namespace['closure'](state.parameters, bool)
 
 
 class State(object):
@@ -348,12 +358,18 @@
     return "NULL"
 
 
-@compile_python.when(str, unicode, bool, int, long, float,
-                     datetime, date, time, timedelta, type(None))
+@compile_python.when(str, unicode, int, long, float, type(None))
 def compile_python_builtin(compile, expr, state):
     return repr(expr)
 
 
+@compile_python.when(bool, datetime, date, time, timedelta)
+def compile_python_bool_and_dates(compile, expr, state):
+    index = len(state.parameters)
+    state.parameters.append(expr)
+    return "_%d" % index
+
+
 @compile.when(Variable)
 def compile_variable(compile, variable, state):
     state.parameters.append(variable)
@@ -361,7 +377,9 @@
 
 @compile_python.when(Variable)
 def compile_python_variable(compile, variable, state):
-    return repr(variable.get())
+    index = len(state.parameters)
+    state.parameters.append(variable.get())
+    return "_%d" % index
 
 
 # --------------------------------------------------------------------
@@ -455,6 +473,9 @@
             other = getattr(self, "variable_factory", Variable)(value=other)
         return Mod(self, other)
 
+    def __neg__(self):
+        return Neg(self)
+
     def is_in(self, others):
         if not isinstance(others, Expr):
             others = list(others)
@@ -776,7 +797,9 @@
 
 @compile_python.when(Column)
 def compile_python_column(compile, column, state):
-    return "get_column(%s)" % repr(column.name)
+    index = len(state.parameters)
+    state.parameters.append(column)
+    return "get_column(_%d)" % index
 
 
 # --------------------------------------------------------------------
@@ -1215,6 +1238,11 @@
     name = "UPPER"
 
 
+class Coalesce(NamedFunc):
+    __slots__ = ()
+    name = "COALESCE"
+
+
 # --------------------------------------------------------------------
 # Prefix and suffix expressions
 
@@ -1250,6 +1278,14 @@
     __slots__ = ()
     prefix = "EXISTS"
 
+class Neg(PrefixExpr):
+    __slots__ = ()
+    prefix = "-"
+
+@compile_python.when(Neg)
+def compile_neg_expr(compile, expr, state):
+    return "-%s" % compile(expr.expr, state, raw=True)
+
 class Asc(SuffixExpr):
     __slots__ = ()
     suffix = "ASC"

=== modified file 'storm/store.py'
--- storm/store.py	2009-02-16 10:44:31 +0000
+++ storm/store.py	2009-05-19 15:17:29 +0000
@@ -1349,11 +1349,8 @@
             match = None
         else:
             match = compile_python.get_matcher(self._where)
-            name_to_column = dict(
-                (column.name, column)
-                for column in self._find_spec.default_cls_info.columns)
-            def get_column(name, name_to_column=name_to_column):
-                return obj_info.variables[name_to_column[name]].get()
+            def get_column(column):
+                return obj_info.variables[column].get()
         objects = []
         cls = self._find_spec.default_cls_info.cls
         for obj_info in self._store._iter_alive():

=== added file 'storm/zope/adapters.py'
--- storm/zope/adapters.py	1970-01-01 00:00:00 +0000
+++ storm/zope/adapters.py	2009-03-05 11:17:47 +0000
@@ -0,0 +1,31 @@
+#
+# Copyright (c) 2006, 2007 Canonical
+#
+# Written by Gustavo Niemeyer <gustavo@niemeyer.net>
+#
+# This file is part of Storm Object Relational Mapper.
+#
+# Storm is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as
+# published by the Free Software Foundation; either version 2.1 of
+# the License, or (at your option) any later version.
+#
+# Storm is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+
+from zope.component import adapter
+from zope.interface import implementer
+
+from storm.zope.interfaces import IResultSet, ISQLObjectResultSet
+
+
+@adapter(ISQLObjectResultSet)
+@implementer(IResultSet)
+def sqlobject_result_set_to_storm_result_set(so_result_set):
+    return so_result_set._result_set

=== modified file 'storm/zope/configure.zcml'
--- storm/zope/configure.zcml	2008-06-09 09:08:19 +0000
+++ storm/zope/configure.zcml	2009-03-05 11:17:47 +0000
@@ -32,4 +32,7 @@
         <require like_class="storm.references.BoundReferenceSet" />
     </class>
 
+    <adapter factory=".adapters.sqlobject_result_set_to_storm_result_set"
+             trusted="yes" />
+
 </configure>

=== modified file 'storm/zope/zstorm.py'
--- storm/zope/zstorm.py	2008-11-28 02:14:00 +0000
+++ storm/zope/zstorm.py	2009-06-26 08:59:26 +0000
@@ -5,7 +5,7 @@
 """
 
 #
-# Copyright (c) 2006, 2007 Canonical
+# Copyright (c) 2006-2009 Canonical
 #
 # Written by Gustavo Niemeyer <gustavo@niemeyer.net>
 #
@@ -144,10 +144,12 @@
         return store
 
     def get(self, name, default_uri=None):
-        """Get the store called C{name} or None if one isn't available.
+        """Get the store called C{name}, creating it first if necessary.
 
         @param default_uri: Optionally, the URI to use to create a
            store called C{name} when one doesn't already exist.
+        @raises ZStormError: Raised if C{uri} is None and no default
+            URI exists for C{name}.
         """
         store = self._named.get(name)
         if not store:

=== modified file 'test'
--- test	2009-02-05 10:16:40 +0000
+++ test	2009-03-05 21:53:12 +0000
@@ -66,7 +66,7 @@
             elif relpath == os.path.join("tests", "zope", "README.txt"):
                 # Special case the inclusion of the Zope-dependent
                 # ZStorm doctest.
-                from tests.zope.zstorm import has_zope
+                from tests.zope import has_zope
                 if has_zope:
                     doctests.append(relpath)
             elif filename.endswith(".txt"):

=== modified file 'tests/database.py'
--- tests/database.py	2008-04-04 23:03:34 +0000
+++ tests/database.py	2009-05-08 17:27:18 +0000
@@ -227,6 +227,54 @@
                                         ("ERROR", self.connection, RawCursor,
                                          "something", (), error)])
 
+    def test_tracing_check_disconnect(self):
+        tracer = FakeTracer()
+        tracer_mock = self.mocker.patch(tracer)
+        tracer_mock.connection_raw_execute(ARGS)
+        self.mocker.throw(DatabaseError('connection closed'))
+        self.mocker.replay()
+
+        install_tracer(tracer_mock)
+        self.connection.is_disconnection_error = (
+            lambda exc: 'connection closed' in str(exc))
+
+        self.assertRaises(DisconnectionError,
+                          self.connection.execute, "something")
+
+    def test_tracing_success_check_disconnect(self):
+        tracer = FakeTracer()
+        tracer_mock = self.mocker.patch(tracer)
+        tracer_mock.connection_raw_execute(ARGS)
+        tracer_mock.connection_raw_execute_success(ARGS)
+        self.mocker.throw(DatabaseError('connection closed'))
+        self.mocker.replay()
+
+        install_tracer(tracer_mock)
+        self.connection.is_disconnection_error = (
+            lambda exc: 'connection closed' in str(exc))
+
+        self.assertRaises(DisconnectionError,
+                          self.connection.execute, "something")
+
+    def test_tracing_error_check_disconnect(self):
+        cursor_mock = self.mocker.patch(RawCursor)
+        cursor_mock.execute(ARGS)
+        error = ZeroDivisionError()
+        self.mocker.throw(error)
+        tracer = FakeTracer()
+        tracer_mock = self.mocker.patch(tracer)
+        tracer_mock.connection_raw_execute(ARGS)
+        tracer_mock.connection_raw_execute_error(ARGS)
+        self.mocker.throw(DatabaseError('connection closed'))
+        self.mocker.replay()
+
+        install_tracer(tracer_mock)
+        self.connection.is_disconnection_error = (
+            lambda exc: 'connection closed' in str(exc))
+
+        self.assertRaises(DisconnectionError,
+                          self.connection.execute, "something")
+
     def test_commit(self):
         self.connection.commit()
         self.assertEquals(self.executed, ["COMMIT"])

=== modified file 'tests/databases/base.py'
--- tests/databases/base.py	2008-10-10 05:44:35 +0000
+++ tests/databases/base.py	2009-07-01 02:48:20 +0000
@@ -33,7 +33,7 @@
 from storm.database import *
 from storm.event import EventSystem
 from storm.exceptions import (
-    DatabaseModuleError, DisconnectionError, OperationalError)
+    DatabaseError, DatabaseModuleError, DisconnectionError, OperationalError)
 
 from tests.databases.proxy import ProxyTCPServer
 from tests.helper import MakePath
@@ -515,6 +515,28 @@
         self.proxy.restart()
         self.assertRaises(DisconnectionError, self.connection.commit)
 
+    def test_wb_catch_already_disconnected(self):
+        """Storm detects connections that have already been disconnected.
+
+        If the connection is being used outside of Storm's control,
+        then it is possible that Storm won't see the disconnection.
+        It should be able to recover from this situation though.
+        """
+        result = self.connection.execute("SELECT 1")
+        self.assertTrue(result.get_one())
+        self.proxy.restart()
+        # Perform an action that should result in a disconnection.
+        try:
+            cursor = self.connection._raw_connection.cursor()
+            cursor.execute("SELECT 1")
+            cursor.fetchone()
+        except DatabaseError, exc:
+            self.assertTrue(self.connection.is_disconnection_error(exc))
+        else:
+            self.fail("Disconnection was not caught.")
+        self.assertRaises(DisconnectionError,
+                          self.connection.execute, "SELECT 1")
+
     def test_connection_stays_disconnected_in_transaction(self):
         """Test that connection does not immediately reconnect."""
         result = self.connection.execute("SELECT 1")

=== modified file 'tests/expr.py'
--- tests/expr.py	2008-10-02 15:47:18 +0000
+++ tests/expr.py	2009-07-23 22:47:10 +0000
@@ -1316,6 +1316,20 @@
         self.assertEquals(statement, "UPPER(func1())")
         self.assertEquals(state.parameters, [])
 
+    def test_coalesce(self):
+        expr = Coalesce(Func1())
+        state = State()
+        statement = compile(expr, state)
+        self.assertEquals(statement, "COALESCE(func1())")
+        self.assertEquals(state.parameters, [])
+
+    def test_coalesce_with_many_arguments(self):
+        expr = Coalesce(Func1(), Func2(), None)
+        state = State()
+        statement = compile(expr, state)
+        self.assertEquals(statement, "COALESCE(func1(), func2(), NULL)")
+        self.assertEquals(state.parameters, [])
+
     def test_not(self):
         expr = Not(Func1())
         state = State()
@@ -1330,6 +1344,19 @@
         self.assertEquals(statement, "EXISTS func1()")
         self.assertEquals(state.parameters, [])
 
+    def test_neg(self):
+        expr = Neg(Func1())
+        state = State()
+        statement = compile(expr, state)
+        self.assertEquals(statement, "- func1()")
+        self.assertEquals(state.parameters, [])
+
+        expr = -Func1()
+        state = State()
+        statement = compile(expr, state)
+        self.assertEquals(statement, "- func1()")
+        self.assertEquals(state.parameters, [])
+
     def test_asc(self):
         expr = Asc(Func1())
         state = State()
@@ -1937,8 +1964,10 @@
 
     def test_compile_sequence(self):
         expr = [elem1, Variable(1), (Variable(2), None)]
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "elem1, 1, 2, None")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "elem1, _0, _1, None")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_compile_invalid(self):
         self.assertRaises(CompileError, compile_python, object())
@@ -1965,8 +1994,10 @@
         self.assertEquals(py_expr, "1L")
 
     def test_bool(self):
-        py_expr = compile_python(True)
-        self.assertEquals(py_expr, "True")
+        state = State()
+        py_expr = compile_python(True, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [True])
 
     def test_float(self):
         py_expr = compile_python(1.1)
@@ -1974,23 +2005,31 @@
 
     def test_datetime(self):
         dt = datetime(1977, 5, 4, 12, 34)
-        py_expr = compile_python(dt)
-        self.assertEquals(py_expr, repr(dt))
+        state = State()
+        py_expr = compile_python(dt, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [dt])
 
     def test_date(self):
         d = date(1977, 5, 4)
-        py_expr = compile_python(d)
-        self.assertEquals(py_expr, repr(d))
+        state = State()
+        py_expr = compile_python(d, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [d])
 
     def test_time(self):
         t = time(12, 34)
-        py_expr = compile_python(t)
-        self.assertEquals(py_expr, repr(t))
+        state = State()
+        py_expr = compile_python(t, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [t])
 
     def test_timedelta(self):
         td = timedelta(days=1, seconds=2, microseconds=3)
-        py_expr = compile_python(td)
-        self.assertEquals(py_expr, repr(td))
+        state = State()
+        py_expr = compile_python(td, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [td])
 
     def test_none(self):
         py_expr = compile_python(None)
@@ -1998,63 +2037,87 @@
 
     def test_column(self):
         expr = Column(column1)
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "get_column('column1')")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "get_column(_0)")
+        self.assertEquals(state.parameters, [expr])
 
     def test_column_table(self):
         expr = Column(column1, table1)
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "get_column('column1')")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "get_column(_0)")
+        self.assertEquals(state.parameters, [expr])
 
     def test_variable(self):
         expr = Variable("value")
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "'value'")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, ["value"])
 
     def test_eq(self):
         expr = Eq(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 == 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 == _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_ne(self):
         expr = Ne(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 != 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 != _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_gt(self):
         expr = Gt(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 > 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 > _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_ge(self):
         expr = Ge(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 >= 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 >= _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_lt(self):
         expr = Lt(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 < 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 < _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_le(self):
         expr = Le(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 <= 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 <= _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_lshift(self):
         expr = LShift(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1<<2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0<<_1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_rshift(self):
         expr = RShift(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1>>2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0>>_1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_in(self):
         expr = In(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 in (2,)")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 in (_1,)")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_and(self):
         expr = And(elem1, elem2, And(elem3, elem4))
@@ -2071,6 +2134,11 @@
         py_expr = compile_python(expr)
         self.assertEquals(py_expr, "elem1+elem2+elem3+elem4")
 
+    def test_neg(self):
+        expr = Neg(elem1)
+        py_expr = compile_python(expr)
+        self.assertEquals(py_expr, "-elem1")
+
     def test_sub(self):
         expr = Sub(elem1, Sub(elem2, elem3))
         py_expr = compile_python(expr)
@@ -2109,8 +2177,20 @@
 
         match = compile_python.get_matcher((col1 > 10) & (col2 < 10))
 
-        self.assertTrue(match({"column1": 15, "column2": 5}.get))
-        self.assertFalse(match({"column1": 5, "column2": 15}.get))
+        self.assertTrue(match({col1: 15, col2: 5}.get))
+        self.assertFalse(match({col1: 5, col2: 15}.get))
+
+    def test_match_bad_repr(self):
+        """The get_matcher() works for expressions containing values
+        whose repr is not valid Python syntax."""
+        class BadRepr(object):
+            def __repr__(self):
+                return "$Not a valid Python expression$"
+
+        value = BadRepr()
+        col1 = Column(column1)
+        match = compile_python.get_matcher(col1 == Variable(value))
+        self.assertTrue(match({col1: value}.get))
 
 
 class LazyValueExprTest(TestHelper):

=== modified file 'tests/zope/README.txt'
--- tests/zope/README.txt	2008-07-24 13:27:05 +0000
+++ tests/zope/README.txt	2009-07-22 20:46:54 +0000
@@ -115,7 +115,7 @@
 
   >>> person = Person(u"John Doe")
   >>> store.add(person)
-  <Person object at ...>
+  <...Person object at ...>
   >>> transaction.commit()
 
 Notice that we're not using store.commit directly; we're using Zope's
@@ -135,7 +135,7 @@
 Let's make sure aborting transactions works, too.
 
   >>> store.add(Person(u"Imposter!"))
-  <Person object at ...>
+  <...Person object at ...>
 
 At this point a store.find should return the new object.
 

=== modified file 'tests/zope/__init__.py'
--- tests/zope/__init__.py	2007-08-05 21:59:54 +0000
+++ tests/zope/__init__.py	2009-03-05 21:53:12 +0000
@@ -17,3 +17,25 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
+
+__all__ = [
+    'has_transaction',
+    'has_zope',
+    'has_zope_component',
+    ]
+
+try:
+    import transaction
+except ImportError:
+    has_transaction = False
+else:
+    has_transaction = True
+
+try:
+    import zope.component
+except ImportError:
+    has_zope_component = False
+else:
+    has_zope_component = True
+
+has_zope = has_transaction and has_zope_component

=== added file 'tests/zope/adapters.py'
--- tests/zope/adapters.py	1970-01-01 00:00:00 +0000
+++ tests/zope/adapters.py	2009-03-05 21:53:12 +0000
@@ -0,0 +1,57 @@
+#
+# Copyright (c) 2006, 2007 Canonical
+#
+# Written by Gustavo Niemeyer <gustavo@niemeyer.net>
+#
+# This file is part of Storm Object Relational Mapper.
+#
+# Storm is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as
+# published by the Free Software Foundation; either version 2.1 of
+# the License, or (at your option) any later version.
+#
+# Storm is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+from tests.helper import TestHelper
+from tests.zope import has_zope_component
+
+
+if has_zope_component:
+    from zope.interface import implements
+    from zope.component import getGlobalSiteManager
+
+    from storm.store import EmptyResultSet
+    from storm.zope.adapters import sqlobject_result_set_to_storm_result_set
+    from storm.zope.interfaces import IResultSet, ISQLObjectResultSet
+
+    class TestSQLObjectResultSet(object):
+        implements(ISQLObjectResultSet)
+        _result_set = EmptyResultSet()
+
+
+class AdaptersTest(TestHelper):
+
+    def is_supported(self):
+        return has_zope_component
+
+    def setUp(self):
+        getGlobalSiteManager().registerAdapter(
+            sqlobject_result_set_to_storm_result_set)
+
+    def tearDown(self):
+        getGlobalSiteManager().unregisterAdapter(
+            sqlobject_result_set_to_storm_result_set)
+
+    def test_adapt_sqlobject_to_storm(self):
+        so_result_set = TestSQLObjectResultSet()
+        self.assertTrue(
+            ISQLObjectResultSet.providedBy(so_result_set))
+        result_set = IResultSet(so_result_set)
+        self.assertTrue(
+            IResultSet.providedBy(result_set))

=== modified file 'tests/zope/zstorm.py'
--- tests/zope/zstorm.py	2008-10-21 09:19:47 +0000
+++ tests/zope/zstorm.py	2009-03-05 21:53:12 +0000
@@ -21,25 +21,15 @@
 import thread, weakref, gc
 
 from tests.helper import TestHelper
+from tests.zope import has_transaction, has_zope_component
 
-try:
+if has_transaction:
     import transaction
-except ImportError:
-    has_transaction = False
-else:
-    has_transaction = True
     from storm.zope.interfaces import IZStorm, ZStormError
     from storm.zope.zstorm import ZStorm, StoreDataManager
 
-try:
+if has_zope_component:
     from zope.component import provideUtility, getUtility
-except ImportError:
-    has_zope_component = False
-else:
-    has_zope_component = True
-
-has_zope = has_transaction and has_zope_component
-
 
 from storm.exceptions import OperationalError
 from storm.locals import Store

